@@ -17,9 +17,16 @@
 /* Define if you have the Andrew File System.  */
 #undef AFS
 
+/* Define if there is a member named d_ino in the struct describing
+   directory headers.  */
+#undef D_INO_IN_DIRENT
+
 /* Define to 1 if NLS is requested.  */
 #undef ENABLE_NLS
 
+/* Define to rpl_fnmatch if the replacement function should be used.  */
+#undef fnmatch
+
 /* Define if your system defines TIOCGWINSZ in sys/ioctl.h.  */
 #undef GWINSZ_IN_SYS_IOCTL
 
@@ -32,19 +39,23 @@
 /* Define if your locale.h file contains LC_MESSAGES.  */
 #undef HAVE_LC_MESSAGES
 
+/* Define to 1 if you have the obstack functions from GNU libc.  */
+#undef HAVE_OBSTACK
+
+/* Define if your struct stat has st_dm_mode.  */
+#undef HAVE_ST_DM_MODE
+
 /* Define to 1 if you have the stpcpy function.  */
 #undef HAVE_STPCPY
 
 /* Define to `unsigned long' if <sys/types.h> doesn't define.  */
 #undef ino_t
 
-/* Define if there is a member named d_ino in the struct describing
-   directory headers.  */
-#undef D_INO_IN_DIRENT
-
 /* Define if `struct utimbuf' is declared -- usually in <utime.h>.  */
 #undef HAVE_STRUCT_UTIMBUF
 
+/* Define to gnu_mktime if the replacement function should be used.  */
+#undef mktime
 
 /* Define one of the following to indicate how a program can
    get a list of mounted filesystems.  */
@@ -91,9 +102,8 @@
 /* Define to the name of the distribution.  */
 #undef PACKAGE
 
-/* The concatenation of the strings PACKAGE, "-", and VERSION.  */
-#undef PACKAGE_VERSION
-
+/* The concatenation of the strings "GNU ", and PACKAGE.  */
+#undef GNU_PACKAGE
 
 /* Define to 1 if ANSI function prototypes are usable.  */
 #undef PROTOTYPES
@@ -136,6 +146,9 @@
 /* Define to the version of the distribution.  */
 #undef VERSION
 
+/* Define to 1 if GNU regex should be used instead of GNU rx.  */
+#undef WITH_REGEX
+
 
 /* Leave that blank line there!!  Autoheader needs it.
    If you're adding to this file, keep in mind:
