@@ -31,15 +31,24 @@
 #include "cp.h"
 #include "backupfile.h"
 #include "argmatch.h"
+#include "path-concat.h"
 
 #ifndef _POSIX_VERSION
 uid_t geteuid ();
 #endif
 
 #ifdef HAVE_LCHOWN
-# define chown(PATH, OWNER, GROUP) lchown(PATH, OWNER, GROUP)
+# define LINK_CHOWN(FILE, OWNER, GROUP) lchown(FILE, OWNER, GROUP)
+#else
+# define LINK_CHOWN(FILE, OWNER, GROUP) chown(FILE, OWNER, GROUP)
 #endif
 
+#define DO_CHOWN(FILE, NEW_UID, NEW_GID)				\
+  (LINK_CHOWN ((FILE), (myeuid == 0 ? (NEW_UID) : myeuid), (NEW_GID))	\
+   /* If non-root uses -p, it's ok if we can't preserve ownership.	\
+      But root probably wants to know, e.g. if NFS disallows it.  */	\
+   && (errno != EPERM || myeuid == 0))
+
 /* Used by do_copy, make_path_private, and re_protect
    to keep a list of leading directories whose protections
    need to be fixed after copying. */
@@ -244,7 +253,7 @@ main (int argc, char **argv)
 	    if (i < 0)
 	      {
 		invalid_arg (_("sparse type"), optarg, i);
-		usage (2, NULL);
+		usage (1);
 	      }
 	    flag_sparse = sparse_type[i];
 	  }
@@ -326,21 +335,24 @@ main (int argc, char **argv)
 	  break;
 
 	default:
-	  usage (2, (char *) 0);
+	  usage (1);
 	}
     }
 
   if (show_version)
     {
-      printf ("cp - %s\n", PACKAGE_VERSION);
+      printf ("cp (%s) %s\n", GNU_PACKAGE, VERSION);
       exit (0);
     }
 
   if (show_help)
-    usage (0, NULL);
+    usage (0);
 
   if (flag_hard_link && flag_symbolic_link)
-    usage (2, _("cannot make both hard and symbolic links"));
+    {
+      error (0, 0, _("cannot make both hard and symbolic links"));
+      usage (1);
+    }
 
   if (make_backups)
     backup_type = get_version (version);
@@ -365,28 +377,6 @@ main (int argc, char **argv)
   exit (exit_status);
 }
 
-/* Concatenate two pathname components, DIR and BASE, in newly-allocated
-   storage and return the result.  Be careful that in the result they are
-   separated by a slash.  That is, if DIR ends with a slash or if BASE
-   begins with one, don't add a separating slash.  Otherwise, add one.  */
-
-static char *
-path_concat (const char *dir, const char *base)
-{
-  char *dir_end;
-  char *p_concat;
-
-  assert (strlen (dir) > 0);
-  p_concat = xmalloc (strlen (dir) + strlen (base) + 2);
-  dir_end = stpcpy (p_concat, dir);
-  if (*(dir_end - 1) == '/')
-    --dir_end;
-  else if (*base == '/')
-    ++base;
-  stpcpy (stpcpy (dir_end, "/"), base);
-  return p_concat;
-}
-
 /* Scan the arguments, and copy each by calling copy.
    Return 0 if successful, 1 if any errors occur. */
 
@@ -399,9 +389,15 @@ do_copy (int argc, char **argv)
   int ret = 0;
 
   if (optind >= argc)
-    usage (2, _("missing file arguments"));
+    {
+      error (0, 0, _("missing file arguments"));
+      usage (1);
+    }
   if (optind >= argc - 1)
-    usage (2, _("missing destination file"));
+    {
+      error (0, 0, _("missing destination file"));
+      usage (1);
+    }
 
   dest = argv[argc - 1];
 
@@ -439,6 +435,7 @@ do_copy (int argc, char **argv)
 	  char *dst_path;
 	  int parent_exists = 1; /* True if dirname (dst_path) exists. */
 	  struct dir_attr *attr_list;
+	  char *arg_in_concat = NULL;
 
 	  arg = argv[optind];
 
@@ -447,13 +444,16 @@ do_copy (int argc, char **argv)
 	  if (flag_path)
 	    {
 	      /* Append all of `arg' to `dest'.  */
-	      dst_path = path_concat (dest, arg);
+	      dst_path = path_concat (dest, arg, &arg_in_concat);
+	      if (dst_path == NULL)
+		error (1, 0, _("virtual memory exhausted"));
 
 	      /* For --parents, we have to make sure that the directory
 	         dirname (dst_path) exists.  We may have to create a few
 	         leading directories. */
 	      parent_exists = !make_path_private (dst_path,
-						  strlen (dest) + 1, 0700,
+						  arg_in_concat - dst_path,
+						  0700,
 						  (flag_verbose
 						   ? "%s -> %s\n" : NULL),
 						  &attr_list, &new_dst);
@@ -466,7 +466,7 @@ do_copy (int argc, char **argv)
 	      /* For `cp -R source/.. dest', don't copy into `dest/..'. */
 	      dst_path = (STREQ (ap, "..")
 			  ? xstrdup (dest)
-			  : path_concat (dest, ap));
+			  : path_concat (dest, ap, NULL));
 	    }
 
 	  if (!parent_exists)
@@ -481,7 +481,8 @@ do_copy (int argc, char **argv)
 
 	      if (flag_path)
 		{
-		  ret |= re_protect (dst_path, strlen (dest) + 1, attr_list);
+		  ret |= re_protect (dst_path, arg_in_concat - dst_path,
+				     attr_list);
 		}
 	    }
 
@@ -499,8 +500,11 @@ do_copy (int argc, char **argv)
       struct stat source_stats;
 
       if (flag_path)
-	usage (2,
+	{
+	  error (0, 0,
 	       _("when preserving paths, last argument must be a directory"));
+	  usage (1);
+	}
 
       source = argv[optind];
 
@@ -514,16 +518,24 @@ do_copy (int argc, char **argv)
 	  && backup_type != none
 	  && STREQ (source, dest)
 	  && !new_dst && S_ISREG (sb.st_mode))
-	 {
-	   backup_type = none;
-	   new_dest = find_backup_file_name (dest);
-	   if (new_dest == NULL)
-	     error (1, 0, _("virtual memory exhausted"));
-	 }
+	{
+	  new_dest = find_backup_file_name (dest);
+	  /* Set backup_type to `none' so that the normal backup
+	     mechanism is not used when performing the actual copy.
+	     backup_type must be set to `none' only *after* the above
+	     call to find_backup_file_name -- that function uses
+	     backup_type to determine the suffix it applies.  */
+	  backup_type = none;
+	  if (new_dest == NULL)
+	    error (1, 0, _("virtual memory exhausted"));
+	}
 
       /* When the destination is specified with a trailing slash and the
 	 source exists but is not a directory, convert the user's command
-	 `cp source dest/' to `cp source dest/basename(source)'.  */
+	 `cp source dest/' to `cp source dest/basename(source)'.  Doing
+	 this ensures that the command `cp non-directory file/' will now
+	 fail rather than performing the copy.  COPY diagnoses the case of
+	 `cp directory non-directory'.  */
 
       else if (dest[strlen (dest) - 1] == '/'
 	  && lstat (source, &source_stats) == 0
@@ -537,9 +549,9 @@ do_copy (int argc, char **argv)
 	  strip_trailing_slashes (tmp_source);
 	  source_base = basename (tmp_source);
 
-	  new_dest = (char *) alloca (strlen (dest) + 1 +
-				      strlen (source_base) + 1);
-	  stpcpy (stpcpy (stpcpy (new_dest, dest), "/"), source_base);
+	  new_dest = (char *) alloca (strlen (dest)
+				      + strlen (source_base) + 1);
+	  stpcpy (stpcpy (new_dest, dest), source_base);
 	}
       else
 	{
@@ -549,8 +561,13 @@ do_copy (int argc, char **argv)
       return copy (source, new_dest, new_dst, 0, (struct dir_list *) 0);
     }
   else
-    usage (2,
-	 _("when copying multiple files, last argument must be a directory"));
+    {
+      error (0, 0,
+	     _("copying multiple files, but last argument (%s) \
+is not a directory"),
+	     dest);
+      usage (1);
+    }
 }
 
 /* Copy the file SRC_PATH to the file DST_PATH.  The files may be of
@@ -644,21 +661,22 @@ copy (const char *src_path, const char *dst_path, int new_dst, dev_t device,
 		return 0;
 	    }
 
-	  if (S_ISREG (src_type) && !flag_force)
+	  if (!S_ISDIR (src_type) && !flag_force && flag_interactive)
 	    {
-	      if (flag_interactive)
+	      if (euidaccess (dst_path, W_OK) != 0)
 		{
-		  if (euidaccess (dst_path, W_OK) != 0)
-		    fprintf (stderr,
-			     _("%s: overwrite `%s', overriding mode %04o? "),
-			     program_name, dst_path,
-			     (unsigned int) (dst_sb.st_mode & 07777));
-		  else
-		    fprintf (stderr, _("%s: overwrite `%s'? "),
-			     program_name, dst_path);
-		  if (!yesno ())
-		    return 0;
+		  fprintf (stderr,
+			   _("%s: overwrite `%s', overriding mode %04o? "),
+			   program_name, dst_path,
+			   (unsigned int) (dst_sb.st_mode & 07777));
 		}
+	      else
+		{
+		  fprintf (stderr, _("%s: overwrite `%s'? "),
+			   program_name, dst_path);
+		}
+	      if (!yesno ())
+		return 0;
 	    }
 
 	  if (backup_type != none && !S_ISDIR (dst_sb.st_mode))
@@ -804,6 +822,18 @@ copy (const char *src_path, const char *dst_path, int new_dst, dev_t device,
 	      error (0, errno, "%s", dst_path);
 	      goto un_backup;
 	    }
+
+	  /* Change the owner and group of the just-created symbolic link
+	     if this system has the lchown function.  */
+#ifdef HAVE_LCHOWN
+	  if (flag_preserve
+	      && DO_CHOWN (dst_path, src_sb.st_uid, src_sb.st_gid))
+	    {
+	      error (0, errno, "%s", dst_path);
+	      goto un_backup;
+	    }
+#endif
+
 	  return 0;
 	}
       else
@@ -879,6 +909,18 @@ copy (const char *src_path, const char *dst_path, int new_dst, dev_t device,
 	  error (0, errno, _("cannot create symbolic link `%s'"), dst_path);
 	  goto un_backup;
 	}
+
+      /* Change the owner and group of the just-created symbolic link
+	 if this system has the lchown function.  */
+#ifdef HAVE_LCHOWN
+      if (flag_preserve
+	  && DO_CHOWN (dst_path, src_sb.st_uid, src_sb.st_gid))
+	{
+	  error (0, errno, "%s", dst_path);
+	  goto un_backup;
+	}
+#endif
+
       return 0;
     }
   else
@@ -905,12 +947,7 @@ copy (const char *src_path, const char *dst_path, int new_dst, dev_t device,
 	  return 1;
 	}
 
-      /* If non-root uses -p, it's ok if we can't preserve ownership.
-	 But root probably wants to know, e.g. if NFS disallows it.  */
-      if (chown (dst_path,
-		 (myeuid == 0 ? src_sb.st_uid : myeuid),
-		 src_sb.st_gid)
-	  && (errno != EPERM || myeuid == 0))
+      if (DO_CHOWN (dst_path, src_sb.st_uid, src_sb.st_gid))
 	{
 	  error (0, errno, "%s", dst_path);
 	  return 1;
