@@ -61,7 +61,7 @@ void hash_init ();
 void strip_trailing_slashes ();
 
 void forget_all __P ((void));
-void usage __P ((int status, const char *reason));
+void usage __P ((int status));
 int is_ancestor __P ((const struct stat *sb,
 		      const struct dir_list *ancestors));
 char *hash_insert __P ((ino_t ino, dev_t dev, const char *node));
