@@ -223,17 +223,19 @@ Summarize disk usage of each FILE, recursively for directories.\n\
   -a, --all             write counts for all files, not just directories\n\
   -b, --bytes           print size in bytes\n\
   -c, --total           produce a grand total\n\
-  -h, --human-readable  print sizes in human readable format (e.g. 1K 234M 2G)\n\
+  -D, --dereference-args  dereference PATHs when symbolic link\n\
+  -h, --human-readable  print sizes in human readable format (e.g., 1K 234M 2G)\n\
   -k, --kilobytes       use 1024-byte blocks, not 512 despite POSIXLY_CORRECT\n\
   -l, --count-links     count sizes many times if hard linked\n\
+  -L, --dereference     dereference all symbolic links\n\
   -m, --megabytes       use 1024K-byte blocks, not 512 despite POSIXLY_CORRECT\n\
+  -S, --separate-dirs   do not include size of subdirectories\n\
   -s, --summarize       display only a total for each argument\n\
   -x, --one-file-system  skip directories on different filesystems\n\
-  -D, --dereference-args  dereference PATHs when symbolic link\n\
-  -L, --dereference     dereference all symbolic links\n\
-  -S, --separate-dirs   do not include size of subdirectories\n\
       --help            display this help and exit\n\
-      --version         output version information and exit\n"));
+      --version         output version information and exit\n\
+"));
+      puts (_("\nReport bugs to fileutils-bugs@gnu.ai.mit.edu"));
     }
   exit (status);
 }
@@ -334,7 +336,7 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("du - %s\n", PACKAGE_VERSION);
+      printf ("du (%s) %s\n", GNU_PACKAGE, VERSION);
       exit (0);
     }
 
