@@ -112,7 +112,7 @@ struct fileinfo
     unsigned int linkmode;
 
     /* For symbolic link and color printing, 1 if linked-to file
-       exits, otherwise 0.  */
+       exists, otherwise 0.  */
     int linkok;
 
     enum filetype filetype;
@@ -662,15 +662,15 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("%s - %s\n",
+      printf ("%s (%s) %s\n",
 	      (ls_mode == LS_LS ? "ls"
 	       : (ls_mode == LS_MULTI_COL ? "dir" : "vdir")),
-	      PACKAGE_VERSION);
-      exit (0);
+	      GNU_PACKAGE, VERSION);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
-    usage (0);
+    usage (EXIT_SUCCESS);
 
   if (print_with_color)
     parse_ls_color ();
@@ -740,6 +740,9 @@ main (int argc, char **argv)
       dired_dump_obstack ("//SUBDIRED//", &subdired_obstack);
     }
 
+  if (fclose (stdout) == EOF)
+    error (EXIT_FAILURE, errno, _("write error"));
+
   exit (exit_status);
 }
 
@@ -948,7 +951,7 @@ decode_switches (int argc, char **argv)
 	case 'w':
 	  if (xstrtol (optarg, NULL, 0, &tmp_long, NULL) != LONGINT_OK
 	      || tmp_long <= 0 || tmp_long > INT_MAX)
-	    error (1, 0, _("invalid line width: %s"), optarg);
+	    error (EXIT_FAILURE, 0, _("invalid line width: %s"), optarg);
 	  line_length = (int) tmp_long;
 	  break;
 
@@ -957,6 +960,7 @@ decode_switches (int argc, char **argv)
 	  break;
 
 	case 'A':
+	  really_all_files = 0;
 	  all_files = 1;
 	  break;
 
@@ -1011,7 +1015,7 @@ decode_switches (int argc, char **argv)
 	case 'T':
 	  if (xstrtol (optarg, NULL, 0, &tmp_long, NULL) != LONGINT_OK
 	      || tmp_long < 0 || tmp_long > INT_MAX)
-	    error (1, 0, _("invalid tab size: %s"), optarg);
+	    error (EXIT_FAILURE, 0, _("invalid tab size: %s"), optarg);
 	  tabsize = (int) tmp_long;
 	  break;
 
@@ -1032,7 +1036,7 @@ decode_switches (int argc, char **argv)
 	  if (i < 0)
 	    {
 	      invalid_arg (_("sort type"), optarg, i);
-	      usage (1);
+	      usage (EXIT_FAILURE);
 	    }
 	  sort_type = sort_types[i];
 	  break;
@@ -1042,7 +1046,7 @@ decode_switches (int argc, char **argv)
 	  if (i < 0)
 	    {
 	      invalid_arg (_("time type"), optarg, i);
-	      usage (1);
+	      usage (EXIT_FAILURE);
 	    }
 	  time_type = time_types[i];
 	  break;
@@ -1052,7 +1056,7 @@ decode_switches (int argc, char **argv)
 	  if (i < 0)
 	    {
 	      invalid_arg (_("format type"), optarg, i);
-	      usage (1);
+	      usage (EXIT_FAILURE);
 	    }
 	  format = formats[i];
 	  break;
@@ -1064,7 +1068,7 @@ decode_switches (int argc, char **argv)
 	      if (i < 0)
 		{
 		  invalid_arg (_("colorization criterion"), optarg, i);
-		  usage (1);
+		  usage (EXIT_FAILURE);
 		}
 	      i = color_types[i];
 	    }
@@ -1089,7 +1093,7 @@ decode_switches (int argc, char **argv)
 	  break;
 
 	default:
-	  usage (1);
+	  usage (EXIT_FAILURE);
 	}
     }
 
@@ -1593,6 +1597,7 @@ gobble_file (const char *name, int explicit_arg, const char *dirname)
 
   files[files_index].linkname = 0;
   files[files_index].linkmode = 0;
+  files[files_index].linkok = 0;
 
   if (explicit_arg || format_needs_stat)
     {
@@ -1658,9 +1663,12 @@ gobble_file (const char *name, int explicit_arg, const char *dirname)
 		  files[files_index].stat = linkstats;
 		}
 	      else
-		/* Get the linked-to file's mode for the filetype indicator
-		   in long listings.  */
-		files[files_index].linkmode = linkstats.st_mode;
+		{
+		  /* Get the linked-to file's mode for the filetype indicator
+		     in long listings.  */
+		  files[files_index].linkmode = linkstats.st_mode;
+		  files[files_index].linkok = 1;
+		}
 	    }
 	  if (linkpath)
 	    free (linkpath);
@@ -2014,7 +2022,8 @@ static void
 print_long_format (const struct fileinfo *f)
 {
   char modebuf[20];
-  char timebuf[40];
+#define TIMEBUF_SIZE 40
+  char timebuf[TIMEBUF_SIZE];
 
   /* 7 fields that may (worst case: 64-bit integral values) require 20 bytes,
      1 10-character mode string,
@@ -2025,7 +2034,12 @@ print_long_format (const struct fileinfo *f)
   char *p;
   time_t when;
 
+#ifdef HAVE_ST_DM_MODE
+  mode_string (f->stat.st_dm_mode, modebuf);
+#else
   mode_string (f->stat.st_mode, modebuf);
+#endif
+
   modebuf[10] = '\0';
 
   switch (time_type)
@@ -2041,7 +2055,9 @@ print_long_format (const struct fileinfo *f)
       break;
     }
 
-  strcpy (timebuf, ctime (&when));
+  /* Use strftime rather than ctime, because the former can produce
+     locale-dependent names for the weekday (%a) and month (%b).  */
+  strftime (timebuf, TIMEBUF_SIZE, "%a %b %d %H:%M:%S %Y", localtime (&when));
 
   if (full_time)
     timebuf[24] = '\0';
@@ -2336,7 +2352,7 @@ print_type_indicator (unsigned int mode)
 #endif
 
   if (S_ISREG (mode) && indicator_style == all
-      && (mode & (S_IEXEC | S_IXGRP | S_IXOTH)))
+      && (mode & S_IXUGO))
     PUTCHAR ('*');
 }
 
@@ -2356,49 +2372,53 @@ print_color_indicator (const char *name, unsigned int mode, int linkok)
     }
   else
     {
-      /* Test if is is a recognized extension.  */
-
-      len = strlen (name);
-      name += len;		/* Pointer to final \0.  */
-      for (ext = col_ext_list; ext != NULL; ext = ext->next)
-	if (ext->ext.len <= len
-	    && strncmp (name - ext->ext.len, ext->ext.string,
-			ext->ext.len) == 0)
-	  break;
-
-      if (ext == NULL)
-	{
-	  if (S_ISDIR (mode))
-	    type = C_DIR;
+      if (S_ISDIR (mode))
+	type = C_DIR;
 
 #ifdef S_ISLNK
-	  else if (S_ISLNK (mode))
-	    type = ((!linkok && color_indicator[C_ORPHAN].string)
-		    ? C_ORPHAN : C_LINK);
+      else if (S_ISLNK (mode))
+	type = ((!linkok && color_indicator[C_ORPHAN].string)
+		? C_ORPHAN : C_LINK);
 #endif
 
 #ifdef S_ISFIFO
-	  else if (S_ISFIFO (mode))
-	    type = C_FIFO;
+      else if (S_ISFIFO (mode))
+	type = C_FIFO;
 #endif
 
 #ifdef S_ISSOCK
-	  else if (S_ISSOCK (mode))
-	    type = C_SOCK;
+      else if (S_ISSOCK (mode))
+	type = C_SOCK;
 #endif
 
 #ifdef S_ISBLK
-	  else if (S_ISBLK (mode))
-	    type = C_BLK;
+      else if (S_ISBLK (mode))
+	type = C_BLK;
 #endif
 
 #ifdef S_ISCHR
-	  else if (S_ISCHR (mode))
-	    type = C_CHR;
+      else if (S_ISCHR (mode))
+	type = C_CHR;
 #endif
 
-	  if (type == C_FILE && (mode & (S_IEXEC|S_IEXEC>>3|S_IEXEC>>6)) != 0)
-	    type = C_EXEC;
+      if (type == C_FILE && (mode & S_IXUGO) != 0)
+	type = C_EXEC;
+
+      /* Check the file's suffix only if still classified as C_FILE.  */
+      ext = NULL;
+      if (type == C_FILE)
+	{
+	  /* Test if NAME has a recognized suffix.  */
+
+	  len = strlen (name);
+	  name += len;		/* Pointer to final \0.  */
+	  for (ext = col_ext_list; ext != NULL; ext = ext->next)
+	    {
+	      if (ext->ext.len <= len
+		  && strncmp (name - ext->ext.len, ext->ext.string,
+			      ext->ext.len) == 0)
+		break;
+	    }
 	}
     }
 
@@ -2477,7 +2497,7 @@ length_of_file_name_and_frills (const struct fileinfo *f)
       if (S_ISREG (filetype))
 	{
 	  if (indicator_style == all
-	      && (f->stat.st_mode & (S_IEXEC | S_IEXEC >> 3 | S_IEXEC >> 6)))
+	      && (f->stat.st_mode & S_IXUGO))
 	    len += 1;
 	}
       else if (S_ISDIR (filetype)
@@ -2689,51 +2709,52 @@ usage (int status)
 List information about the FILEs (the current directory by default).\n\
 Sort entries alphabetically if none of -cftuSUX nor --sort.\n\
 \n\
-  -A, --almost-all           do not list implied . and ..\n\
   -a, --all                  do not hide entries starting with .\n\
-  -B, --ignore-backups       do not list implied entries ending with ~\n\
+  -A, --almost-all           do not list implied . and ..\n\
   -b, --escape               print octal escapes for nongraphic characters\n\
-  -C                         list entries by columns\n\
+  -B, --ignore-backups       do not list implied entries ending with ~\n\
   -c                         sort by change time; with -l: show ctime\n\
+  -C                         list entries by columns\n\
       --color[=WHEN]         control whether color is used to distinguish file\n\
                                types.  WHEN may be `never', `always', or `auto'\n\
-  -D, --dired                generate output designed for Emacs' dired mode\n\
   -d, --directory            list directory entries instead of contents\n\
-  -F, --classify             append a character for typing each entry\n\
+  -D, --dired                generate output designed for Emacs' dired mode\n\
   -f                         do not sort, enable -aU, disable -lst\n\
+  -F, --classify             append a character for typing each entry\n\
       --format=WORD          across -x, commas -m, horizontal -x, long -l,\n\
                                single-column -1, verbose -l, vertical -C\n\
       --full-time            list both full date and full time\n"));
 
       printf (_("\
-  -G, --no-group             inhibit display of group information\n\
   -g                         (ignored)\n\
-  -I, --ignore=PATTERN       do not list implied entries matching shell PATTERN\n\
+  -G, --no-group             inhibit display of group information\n\
   -i, --inode                print index number of each file\n\
+  -I, --ignore=PATTERN       do not list implied entries matching shell PATTERN\n\
   -k, --kilobytes            use 1024 blocks, not 512 despite POSIXLY_CORRECT\n\
-  -L, --dereference          list entries pointed to by symbolic links\n\
   -l                         use a long listing format\n\
+  -L, --dereference          list entries pointed to by symbolic links\n\
   -m                         fill width with a comma separated list of entries\n\
+  -n, --numeric-uid-gid      list numeric UIDs and GIDs instead of names\n\
   -N, --literal              print raw entry names (don't treat e.g. control\n\
                                characters specially)\n\
-  -n, --numeric-uid-gid      list numeric UIDs and GIDs instead of names\n\
   -o                         use long listing format without group info\n\
   -p                         append a character for typing each entry\n\
-  -Q, --quote-name           enclose entry names in double quotes\n\
   -q, --hide-control-chars   print ? instead of non graphic characters\n\
-  -R, --recursive            list subdirectories recursively\n\
+  -Q, --quote-name           enclose entry names in double quotes\n\
   -r, --reverse              reverse order while sorting\n\
-  -S                         sort by file size\n"));
+  -R, --recursive            list subdirectories recursively\n\
+  -s, --size                 print size of each file, in blocks\n"));
 
       printf (_("\
-  -s, --size                 print block size of each file\n\
+  -S                         sort by file size\n\
       --sort=WORD            ctime -c, extension -X, none -U, size -S,\n\
-                               status -c, time -t\n\
-      --time=WORD            atime -u, access -u, use -u\n\
-  -T, --tabsize=COLS         assume tab stops at each COLS instead of 8\n\
+                             status -c, time -t, atime -u, access -u, use -u\n\
+      --time=WORD            show time as WORD instead of modification time:\n\
+                             atime, access, use, ctime or status\n\
   -t                         sort by modification time; with -l: show mtime\n\
-  -U                         do not sort; list entries in directory order\n\
+  -T, --tabsize=COLS         assume tab stops at each COLS instead of 8\n\
   -u                         sort by last access time; with -l: show atime\n\
+  -U                         do not sort; list entries in directory order\n\
   -w, --width=COLS           assume screen width instead of current value\n\
   -x                         list entries by lines instead of by columns\n\
   -X                         sort alphabetically by entry extension\n\
@@ -2746,8 +2767,8 @@ equivalent to using --color=none.  Using the --color option without the\n\
 optional WHEN argument is equivalent to using --color=always.  With\n\
 --color=auto, color codes are output only if standard output is connected\n\
 to a terminal (tty).\n\
-"
-		));
+"));
+      puts (_("\nReport bugs to fileutils-bugs@gnu.ai.mit.edu"));
     }
   exit (status);
 }
